---
title: Ast Services
author:
date: 2024-06-06
...

# Ast Services

The Frama-C internal representation of source files need to be computed before
being accessed.  This generally involves preprocessing of sources and finally
parsing, typechecking and normalization.

Although this step is generally performed silently on-demand inside Frama-C,
from the Server point of view, this is a non-trivial procedure that should _not_ be
triggered outside an `EXEC` request.

Hence, most AST services might fail or return empty data if the AST has not been
actually computed before. However, in typical usage of Frama-C from the
command-line, the AST would have been computed just before any other plug-in,
including the Server.

## kernel.ast.compute (`EXEC`) {#compute}

Ensures that AST is computed


> _input_ `::=` _null_
  

> _output_ `::=` _null_
  

## kernel.ast.changed (`SIGNAL`) {#changed}

Emitted when the AST has been changed


## kernel.ast.source (`DATA`) {#source}

Source file positions.


> [`source`](#source)  `::=` `{` `"dir"` `:` _string_ , `"base"` `:` _string_
  , `"file"` `:` _string_ , `"line"` `:` _number_ `}`
  

## kernel.ast.decl (`DATA`) {#decl}

AST Declarations markers


> [`decl`](#decl)  `::=` `$decl`
  

## kernel.ast.marker (`DATA`) {#marker}

Localizable AST markers


> [`marker`](#marker)  `::=` `$marker`
  

## kernel.ast.declKind (`DATA`) {#declkind}

Declaration kind


> [`declKind`](#declkind)  `::=` `$ENUM` | `$UNION` | `$STRUCT` | `$TYPEDEF`
  | `$GLOBAL` | `$FUNCTION`
  

## kernel.ast.declAttributes (`ARRAY`) {#declattributes}

Declaration attributes


## kernel.ast.signalDeclAttributes (`SIGNAL`) {#signaldeclattributes}

Signal for array [`declAttributes`](#declattributes) 


## kernel.ast.declAttributesData (`DATA`) {#declattributesdata}

Data for array rows [`declAttributes`](#declattributes) 


> [`declAttributesData`](#declattributesdata)  `::= {` _fields…_ `}`
  

| Field | Format | Description |
|:------|:------:|:------------|
| `"decl"` | [`decl`](ast.md#decl)  | Entry identifier. |
| `"kind"` | [`declKind`](ast.md#declkind)  | Declaration kind |
| `"self"` | [`marker`](ast.md#marker)  | Declaration's marker |
| `"name"` | _string_ | Declaration identifier |
| `"label"` | _string_ | Declaration label (uncapitalized kind & name) |
| `"source"` | [`source`](ast.md#source)  | Source location |


## kernel.ast.fetchDeclAttributes (`GET`) {#fetchdeclattributes}

Data fetcher for array [`declAttributes`](#declattributes) 


> _input_ `::=` _number_
  

> _output_ `::=` `{` _output…_ `}`
  

| Output | Format | Description |
|:-------|:------:|:------------|
| `"reload"` | _boolean_ | array fully reloaded |
| `"removed"` | [`decl`](ast.md#decl)  `[]` | removed entries |
| `"updated"` | [`declAttributesData`](ast.md#declattributesdata)  `[]` | updated entries |
| `"pending"` | _number_ | remaining entries to be fetched |


## kernel.ast.reloadDeclAttributes (`GET`) {#reloaddeclattributes}

Force full reload for array [`declAttributes`](#declattributes) 


> _input_ `::=` _null_
  

> _output_ `::=` _null_
  

## kernel.ast.printDeclaration (`GET`) {#printdeclaration}

Prints an AST Declaration


> _input_ `::=` [`decl`](ast.md#decl) 
  

> _output_ `::=` [`text`](data.md#text) 
  

> _signals_
  

* `kernel.ast.changed`


## kernel.ast.markerKind (`DATA`) {#markerkind}

Marker kind


> [`markerKind`](#markerkind)  `::=` `$STMT` | `$LFUN` | `$DFUN` | `$LVAR` |
  `$DVAR` | `$LVAL` | `$EXP` | `$TERM` | `$TYPE` | `$PROPERTY` |
  `$DECLARATION`
  

## kernel.ast.markerAttributes (`ARRAY`) {#markerattributes}

Marker attributes


## kernel.ast.signalMarkerAttributes (`SIGNAL`) {#signalmarkerattributes}

Signal for array [`markerAttributes`](#markerattributes) 


## kernel.ast.markerAttributesData (`DATA`) {#markerattributesdata}

Data for array rows [`markerAttributes`](#markerattributes) 


> [`markerAttributesData`](#markerattributesdata)  `::= {` _fields…_ `}`
  

| Field | Format | Description |
|:------|:------:|:------------|
| `"marker"` | [`marker`](ast.md#marker)  | Entry identifier. |
| `"kind"` | [`markerKind`](ast.md#markerkind)  | Marker kind (key) |
| `"scope"` (opt.) | [`decl`](ast.md#decl)  | Marker Scope (where it is printed in) |
| `"definition"` (opt.) | [`marker`](ast.md#marker)  | Marker's Target Definition (when applicable) |
| `"labelKind"` | _string_ | Marker kind label |
| `"titleKind"` | _string_ | Marker kind title |
| `"name"` (opt.) | _string_ | Marker identifier (when applicable) |
| `"descr"` | _string_ | Marker description |
| `"sloc"` (opt.) | [`source`](ast.md#source)  | Source location |


## kernel.ast.fetchMarkerAttributes (`GET`) {#fetchmarkerattributes}

Data fetcher for array [`markerAttributes`](#markerattributes) 


> _input_ `::=` _number_
  

> _output_ `::=` `{` _output…_ `}`
  

| Output | Format | Description |
|:-------|:------:|:------------|
| `"reload"` | _boolean_ | array fully reloaded |
| `"removed"` | [`marker`](ast.md#marker)  `[]` | removed entries |
| `"updated"` | [`markerAttributesData`](ast.md#markerattributesdata)  `[]` | updated entries |
| `"pending"` | _number_ | remaining entries to be fetched |


## kernel.ast.reloadMarkerAttributes (`GET`) {#reloadmarkerattributes}

Force full reload for array [`markerAttributes`](#markerattributes) 


> _input_ `::=` _null_
  

> _output_ `::=` _null_
  

## kernel.ast.getMainFunction (`GET`) {#getmainfunction}

Get the current 'main' function.


> _input_ `::=` _null_
  

> _output_ `::=` [`decl`](ast.md#decl)  `?`
  

## kernel.ast.getFunctions (`GET`) {#getfunctions}

Collect all functions in the AST


> _input_ `::=` _null_
  

> _output_ `::=` [`decl`](ast.md#decl)  `[]`
  

## kernel.ast.functions (`ARRAY`) {#functions}

AST Functions


## kernel.ast.signalFunctions (`SIGNAL`) {#signalfunctions}

Signal for array [`functions`](#functions) 


## kernel.ast.functionsData (`DATA`) {#functionsdata}

Data for array rows [`functions`](#functions) 


> [`functionsData`](#functionsdata)  `::= {` _fields…_ `}`
  

| Field | Format | Description |
|:------|:------:|:------------|
| `"key"` | `$functions` | Entry identifier. |
| `"decl"` | [`decl`](ast.md#decl)  | Declaration Tag |
| `"name"` | _string_ | Name |
| `"signature"` | _string_ | Signature |
| `"main"` (opt.) | _boolean_ | Is the function the main entry point |
| `"defined"` (opt.) | _boolean_ | Is the function defined? |
| `"stdlib"` (opt.) | _boolean_ | Is the function from the Frama-C stdlib? |
| `"builtin"` (opt.) | _boolean_ | Is the function a Frama-C builtin? |
| `"extern"` (opt.) | _boolean_ | Is the function extern? |
| `"sloc"` | [`source`](ast.md#source)  | Source location |


## kernel.ast.fetchFunctions (`GET`) {#fetchfunctions}

Data fetcher for array [`functions`](#functions) 


> _input_ `::=` _number_
  

> _output_ `::=` `{` _output…_ `}`
  

| Output | Format | Description |
|:-------|:------:|:------------|
| `"reload"` | _boolean_ | array fully reloaded |
| `"removed"` | `$functions` `[]` | removed entries |
| `"updated"` | [`functionsData`](ast.md#functionsdata)  `[]` | updated entries |
| `"pending"` | _number_ | remaining entries to be fetched |


## kernel.ast.reloadFunctions (`GET`) {#reloadfunctions}

Force full reload for array [`functions`](#functions) 


> _input_ `::=` _null_
  

> _output_ `::=` _null_
  

## kernel.ast.globals (`ARRAY`) {#globals}

AST global variables


## kernel.ast.signalGlobals (`SIGNAL`) {#signalglobals}

Signal for array [`globals`](#globals) 


## kernel.ast.globalsData (`DATA`) {#globalsdata}

Data for array rows [`globals`](#globals) 


> [`globalsData`](#globalsdata)  `::= {` _fields…_ `}`
  

| Field | Format | Description |
|:------|:------:|:------------|
| `"key"` | `$globals` | Entry identifier. |
| `"decl"` | [`decl`](ast.md#decl)  | Declaration Tag |
| `"name"` | _string_ | Name |
| `"type"` | _string_ | Type |
| `"stdlib"` | _boolean_ | Is the variable from the Frama-C stdlib? |
| `"extern"` | _boolean_ | Is the variable extern? |
| `"const"` | _boolean_ | Is the variable const? |
| `"volatile"` | _boolean_ | Is the variable volatile? |
| `"ghost"` | _boolean_ | Is the variable ghost? |
| `"init"` | _boolean_ | Is the variable explicitly initialized? |
| `"source"` | _boolean_ | Is the variable in the source code? |
| `"sloc"` | [`source`](ast.md#source)  | Source location |


## kernel.ast.fetchGlobals (`GET`) {#fetchglobals}

Data fetcher for array [`globals`](#globals) 


> _input_ `::=` _number_
  

> _output_ `::=` `{` _output…_ `}`
  

| Output | Format | Description |
|:-------|:------:|:------------|
| `"reload"` | _boolean_ | array fully reloaded |
| `"removed"` | `$globals` `[]` | removed entries |
| `"updated"` | [`globalsData`](ast.md#globalsdata)  `[]` | updated entries |
| `"pending"` | _number_ | remaining entries to be fetched |


## kernel.ast.reloadGlobals (`GET`) {#reloadglobals}

Force full reload for array [`globals`](#globals) 


> _input_ `::=` _null_
  

> _output_ `::=` _null_
  

## kernel.ast.getInformationUpdate (`SIGNAL`) {#getinformationupdate}

Updated AST information


## kernel.ast.getInformation (`GET`) {#getinformation}

Get available information about markers. When no marker is given, returns all kinds of information (with empty `descr` field).


> _input_ `::=` [`marker`](ast.md#marker)  `?`
  

> _output_ `::=` `{` `"id"` `:` _string_ , `"label"` `:` _string_ , `"title"`
  `:` _string_ , `"descr"` `:` _string_ , `"text"` `:` [`text`](data.md#text)
   `}` `[]`
  

> _signals_
  

* `kernel.ast.getInformationUpdate`


## kernel.ast.getMarkerAt (`GET`) {#getmarkerat}

Returns the marker and function at a source file position, if any. Input: file path, line and column. File can be empty, in case no marker is returned.


> _input_ `::=` `{` _input…_ `}`
  

> _output_ `::=` [`marker`](ast.md#marker)  `?`
  

| Input | Format | Description |
|:------|:------:|:------------|
| `"file"` | _string_ | File path |
| `"line"` | _number_ | Line (1-based) |
| `"column"` | _number_ | Column (0-based) |


> _signals_
  

* `kernel.ast.changed`


## kernel.ast.getFiles (`GET`) {#getfiles}

Get the currently analyzed source file names


> _input_ `::=` _null_
  

> _output_ `::=` _string_ `[]`
  

## kernel.ast.setFiles (`SET`) {#setfiles}

Set the source file names to analyze.


> _input_ `::=` _string_ `[]`
  

> _output_ `::=` _null_
  

## kernel.ast.parseExpr (`GET`) {#parseexpr}

Parse a C expression and returns the associated marker


> _input_ `::=` `{` _input…_ `}`
  

> _output_ `::=` [`marker`](ast.md#marker) 
  

| Input | Format | Description |
|:------|:------:|:------------|
| `"stmt"` | [`marker`](ast.md#marker)  | Marker position from where to localize the term |
| `"term"` | _string_ | ACSL term to parse |


## kernel.ast.parseLval (`GET`) {#parselval}

Parse a C lvalue and returns the associated marker


> _input_ `::=` `{` _input…_ `}`
  

> _output_ `::=` [`marker`](ast.md#marker) 
  

| Input | Format | Description |
|:------|:------:|:------------|
| `"stmt"` | [`marker`](ast.md#marker)  | Marker position from where to localize the term |
| `"term"` | _string_ | ACSL term to parse |
